# Generated by Django 3.2.10 on 2022-01-25 09:42

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("buybackprogram", "0002_contractnotification"),
    ]

    operations = [
        migrations.AddField(
            model_name="program",
            name="name",
            field=models.CharField(
                blank=True,
                default="",
                help_text="A name or a description for this program",
                max_length=64,
                verbose_name="Name/description",
            ),
        ),
        migrations.AddField(
            model_name="tracking",
            name="contract",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="buybackprogram.contract",
            ),
        ),
        migrations.AddField(
            model_name="tracking",
            name="created_at",
            field=models.DateTimeField(blank=True, null=True),
        ),
    ]
